@@ -142,8 +142,8 @@ void KHangMan::slotChangeLevel()
     static const char *levelStrings[] = {
         I18N_NOOP("Animals"),
         I18N_NOOP("Easy"),
-        I18N_NOOP("Hard"),
         I18N_NOOP("Medium"),
+        I18N_NOOP("Hard"),
     };
     currentLevel = m_levelAction->currentItem();
     levelString = levels[currentLevel];
@@ -234,7 +234,7 @@ void KHangMan::setLanguages()
     for (uint i=0;  i<m_languages.count(); i++) {
         QString tmp = m_languages[i];
         if (!config->readEntry(tmp))
-            config->writeEntry(tmp, QDate::currentDate().toString());
+            config->writeEntry(tmp, QDate::currentDate().toString(Qt::ISODate));
     }
 
     // We look in $KDEDIR/share/locale/all_languages from
@@ -310,7 +310,18 @@ void KHangMan::loadLevels()
     }
     //TODO else tell no files had been found
     }
+
+    // Sort easy, medium, hard at bottom, with the other categories at the top
     levels.sort();
+    QString replace[3] = { "Easy", "Medium", "Hard" };
+    for ( int i = 0; i < 3; ++i )
+    {
+        if ( levels.findIndex( replace[i] ) > -1 )
+        {
+            levels.remove( replace[i] );
+            levels.append( replace[i] );
+        }
+    }
 
     //find duplicated entries in KDEDIR and KDEHOME
     QString last;
