@@ -33,6 +33,7 @@
 #include "skyobject.h"
 #include "timedialog.h"
 #include "geolocation.h"
+#include "indi/base64.h"
 
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -41,6 +42,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <termios.h>
+#include <zlib.h>
 
 #include <qlineedit.h>
 #include <qtextedit.h>
@@ -75,11 +77,11 @@
 #include <kstatusbar.h>
 #include <kpopupmenu.h>
 
-#define NINDI_STD	13
+#define NINDI_STD	26
 /* INDI standard property used across all clients to enable interoperability. */
+
 const char * indi_std[NINDI_STD] = 
-  {"CONNECTION", "EQUATORIAL_COORD", "ON_COORD_SET", "ABORT_MOTION", "SOLAR_SYSTEM",
-   "GEOGRAPHIC_COORD", "HORIZONTAL_COORD", "TIME", "EXPOSE_DURATION", "DEVICE_PORT", "PARK", "MOVEMENT", "SDTIME"};
+  {"CONNECTION", "DEVICE_PORT", "TIME", "SDTIME", "GEOGRAPHIC_COORD", "EQUATORIAL_COORD", "EQUATORIAL_EOD_COORD", "HORIZONTAL_COORD", "ABORT_MOTION", "ON_COORD_SET", "SOLAR_SYSTEM", "MOVEMENT", "PARK", "CCD_EXPOSE_DURATION", "CCD_TEMPERATURE", "CCD_FRAME", "CCD_FRAME_TYPE", "CCD_BINNING", "CCD_INFO", "CCDPREVIEW_STREAM", "CCDPREVIEW_CTRL", "VIDEO_STREAM", "FOCUS_SPEED", "FOCUS_MOTION", "FOCUS_TIMER", "FILTER_SLOT" };
 
 /*******************************************************************
 ** INDI Device: The work-horse. Responsible for handling its
@@ -100,6 +102,8 @@ INDI_D::INDI_D(INDIMenu *menuParent, DeviceManager *parentManager, QString inNam
  msgST_w        = new QTextEdit(deviceVBox);
  msgST_w->setReadOnly(true);
  msgST_w->setMaximumHeight(100);
+
+ dataBuffer = (unsigned char *) malloc (1);
  
  stdDev 	= new INDIStdDevice(this, parent->ksw);
 
@@ -114,6 +118,10 @@ INDI_D::~INDI_D()
    gl.clear();
    delete(deviceVBox);
    delete (stdDev);
+   free (dataBuffer);
+   dataBuffer = NULL;
+   deviceVBox = NULL;
+   stdDev     = NULL;
 }
 
 void INDI_D::registerProperty(INDI_P *pp)
@@ -170,7 +178,7 @@ int INDI_D::setAnyCmd (XMLEle *root, char errmsg[])
 	pp = findProp (valuXMLAtt(ap));
 	if (!pp)
 	{
-	    sprintf (errmsg,"INDI: <%s> device %s has no property named %s",
+	    snprintf (errmsg, ERRMSG_SIZE, "INDI: <%.32s> device %.32s has no property named %.64s",
 						tagXMLEle(root), name.ascii(), valuXMLAtt(ap));
 	    return (-1);
 	}
@@ -195,7 +203,7 @@ int INDI_D::setValue (INDI_P *pp, XMLEle *root, char errmsg[])
 	      pp->drawLt (pp->state);
 	    else
 	    {
-		sprintf (errmsg, "INDI: <%s> bogus state %s for %s %s",
+		snprintf (errmsg, ERRMSG_SIZE, "INDI: <%.64s> bogus state %.64s for %.64s %.64s",
 						tagXMLEle(root), valuXMLAtt(ap), name.ascii(), pp->name.ascii());
 		return (-1);
 	    }
@@ -223,6 +231,10 @@ int INDI_D::setValue (INDI_P *pp, XMLEle *root, char errmsg[])
 	case PG_MENU:
 	    return (setLabelState (pp, root, errmsg));
 	    break;
+	    
+	case PG_BLOB:
+	    return (setBLOB(pp, root, errmsg));
+	    break;
 
 	default:
 	    break;
@@ -243,6 +255,7 @@ int INDI_D::setTextValue (INDI_P *pp, XMLEle *root, char errmsg[])
 	INDI_E *lp;
 	QString elementName;
 	char iNumber[32];
+	double min, max;
 	
 	for (ep = nextXMLEle (root, 1); ep != NULL; ep = nextXMLEle (root, 0))
 	{
@@ -262,7 +275,7 @@ int INDI_D::setTextValue (INDI_P *pp, XMLEle *root, char errmsg[])
 	    
 	    if (!lp)
 	    {
-	      sprintf(errmsg, "Error: unable to find element '%s' in property '%s'", elementName.ascii(), pp->name.ascii());
+	      snprintf(errmsg, ERRMSG_SIZE, "Error: unable to find element '%.64s' in property '%.64s'", elementName.ascii(), pp->name.ascii());
 	      return (-1);
 	    }
 	    
@@ -282,6 +295,18 @@ int INDI_D::setTextValue (INDI_P *pp, XMLEle *root, char errmsg[])
 	       numberFormat(iNumber, lp->format.ascii(), lp->value);
 	       lp->text = iNumber;
 	       lp->read_w->setText(lp->text);
+	       
+	       ap = findXMLAtt (ep, "min");
+	       if (ap) { min = atof(valuXMLAtt(ap)); lp->setMin(min); }
+	       ap = findXMLAtt (ep, "max");
+	       if (ap) { max = atof(valuXMLAtt(ap)); lp->setMax(max); }
+	       
+	       /*if (lp->spin_w)
+	       {
+	        lp->spin_w->setValue(lp->value);
+		lp->spinChanged(lp->value);
+	     }*/
+	      
 	     }
 	     break;
 
@@ -298,6 +323,11 @@ int INDI_D::setTextValue (INDI_P *pp, XMLEle *root, char errmsg[])
                 lp->spin_w->setValue(lp->value);
 	      else
  	        lp->write_w->setText(lp->text);
+		
+	       ap = findXMLAtt (ep, "min");
+	       if (ap) { min = (int) atof(valuXMLAtt(ap)); lp->setMin(min); }
+	       ap = findXMLAtt (ep, "max");
+	       if (ap) { max = (int) atof(valuXMLAtt(ap)); lp->setMax(max); }
 	    }
 	    break;
 
@@ -340,7 +370,7 @@ int INDI_D::setLabelState (INDI_P *pp, XMLEle *root, char errmsg[])
 	    /* no name */
 	    if (!ap)
 	    {
-		sprintf (errmsg, "INDI: <%s> %s %s %s requires name",
+		snprintf (errmsg, ERRMSG_SIZE, "INDI: <%.64s> %.64s %.64s %.64s requires name",
 						    tagXMLEle(root), name.ascii(), pp->name.ascii(), tagXMLEle(ep));
 		return (-1);
 	    }
@@ -348,7 +378,7 @@ int INDI_D::setLabelState (INDI_P *pp, XMLEle *root, char errmsg[])
 	    if ((islight && crackLightState (pcdataXMLEle(ep), &state) < 0)
 		    || (!islight && crackSwitchState (pcdataXMLEle(ep), &state) < 0))
 	    {
-		sprintf (errmsg, "INDI: <%s> unknown state %s for %s %s %s",
+		snprintf (errmsg, ERRMSG_SIZE, "INDI: <%.64s> unknown state %.64s for %.64s %.64s %.64s",
 					    tagXMLEle(root), pcdataXMLEle(ep), name.ascii(), pp->name.ascii(), tagXMLEle(ep));
 		return (-1);
 	    }
@@ -359,7 +389,7 @@ int INDI_D::setLabelState (INDI_P *pp, XMLEle *root, char errmsg[])
 
 	    if (!lp)
 	    {
-		sprintf (errmsg,"INDI: <%s> %s %s has no choice named %s",
+		snprintf (errmsg, ERRMSG_SIZE, "INDI: <%.64s> %.64s %.64s has no choice named %.64s",
 						    tagXMLEle(root), name.ascii(), pp->name.ascii(), valuXMLAtt(ap));
 		return (-1);
 	    }
@@ -389,7 +419,7 @@ int INDI_D::setLabelState (INDI_P *pp, XMLEle *root, char errmsg[])
 	       {
 	      	if (menuChoice)
 	      	{
-	        	sprintf(errmsg, "INDI: <%s> %s %s has multiple ON states", tagXMLEle(root), name.ascii(), pp->name.ascii());
+	        	snprintf(errmsg, ERRMSG_SIZE, "INDI: <%.64s> %.64s %.64s has multiple ON states", tagXMLEle(root), name.ascii(), pp->name.ascii());
 			return (-1);
               	}
 	      	menuChoice = 1;
@@ -412,6 +442,126 @@ int INDI_D::setLabelState (INDI_P *pp, XMLEle *root, char errmsg[])
 	return (0);
 }
 
+/* Set BLOB vector. Process incoming data stream
+ * Return 0 if okay, -1 if error 
+*/
+int INDI_D::setBLOB(INDI_P *pp, XMLEle * root, char errmsg[])
+{
+  
+  XMLEle *ep;
+  INDI_E *blobEL;
+  
+  for (ep = nextXMLEle(root,1); ep; ep = nextXMLEle(root,0))
+  {
+    
+    if (strcmp(tagXMLEle(ep), "oneBLOB") == 0)
+    {
+      
+      blobEL = pp->findElement(QString(findXMLAttValu (ep, "name")));
+      
+      if (blobEL)
+	return processBlob(blobEL, ep, errmsg);
+      else
+      {
+	sprintf (errmsg, "INDI: set %64s.%64s.%64s not found", name.ascii(), pp->name.ascii(), findXMLAttValu(ep, "name"));
+	return (-1);
+      }
+    }
+  }
+  
+  return (0);
+  
+}
+
+/* Process incoming data stream
+ * Return 0 if okay, -1 if error 
+*/
+int INDI_D::processBlob(INDI_E *blobEL, XMLEle *ep, char errmsg[])
+{
+  XMLAtt *ap;
+  int blobSize=0, r=0, dataType=0;
+  uLongf dataSize=0;
+  QString dataFormat;
+  char *baseBuffer;
+  unsigned char *blobBuffer(NULL);
+  bool iscomp(false);
+  
+  ap = findXMLAtt(ep, "size");
+  if (!ap)
+  {
+    sprintf (errmsg, "INDI: set %64s size not found", blobEL->name.ascii());
+    return (-1);
+  }
+  
+  dataSize = atoi(valuXMLAtt(ap));
+  
+  ap = findXMLAtt(ep, "format");
+  if (!ap)
+  {
+    sprintf (errmsg, "INDI: set %64s format not found", blobEL->name.ascii());
+    return (-1);
+  }
+  
+  dataFormat = QString(valuXMLAtt(ap));
+  
+  baseBuffer = (char *) malloc ( (3*pcdatalenXMLEle(ep)/4) * sizeof (char));
+  blobSize   = from64tobits (baseBuffer, pcdataXMLEle(ep));
+  blobBuffer = (unsigned char *) baseBuffer;
+  
+  /* Blob size = 0 when only state changes */
+  if (dataSize == 0)
+  {
+    free (blobBuffer);
+    return (0);
+  }
+  else if (blobSize < 0)
+  {
+    free (blobBuffer);
+    sprintf (errmsg, "INDI: %64s.%64s.%64s bad base64", name.ascii(), blobEL->pp->name.ascii(), blobEL->name.ascii());
+    return (-1);
+  }
+  
+  iscomp = (dataFormat.find(".z") != -1);
+  
+  dataFormat.remove(".z");
+  
+  if (dataFormat == ".fits") dataType = DATA_FITS;
+  else if (dataFormat == ".stream") dataType = DATA_STREAM;
+  else if (dataFormat == ".ccdpreview") dataType = DATA_CCDPREVIEW;	  
+  else dataType = DATA_OTHER;
+  
+  //kdDebug() << "We're getting data with size " << dataSize << endl;
+  //kdDebug() << "data format " << dataFormat << endl;
+
+  if (iscomp)
+  {
+    
+    dataBuffer = (unsigned char *) realloc (dataBuffer,  (dataSize * sizeof(unsigned char)));
+    r = uncompress(dataBuffer, &dataSize, blobBuffer, (uLong) blobSize);
+    if (r != Z_OK)
+    {
+      sprintf(errmsg, "INDI: %64s.%64s.%64s compression error: %d", name.ascii(), blobEL->pp->name.ascii(), blobEL->name.ascii(), r);    
+      free (blobBuffer);
+      return -1;
+    }
+
+    //kdDebug() << "compressed" << endl;
+  }
+  else
+  {
+    //kdDebug() << "uncompressed!!" << endl;
+    dataBuffer = (unsigned char *) realloc (dataBuffer,  (dataSize * sizeof(unsigned char)));
+    memcpy(dataBuffer, blobBuffer, dataSize);
+  }
+  
+  stdDev->handleBLOB(dataBuffer, dataSize, dataFormat);
+
+  free (blobBuffer);
+  
+  return (0);
+  
+}
+
 bool INDI_D::isOn()
 {
 
@@ -442,11 +592,6 @@ INDI_P * INDI_D::addProperty (XMLEle *root, char errmsg[])
 
 	if (!pg)
 	 return NULL;
-	 
-	 /* Remove Vertical spacer from group layout, this is done everytime
-	  * a new property arrives. The spacer is then appended to the end of the 
-	  * properties */
-	pg->propertyLayout->removeItem(pg->VerticalSpacer);
 
         /* get property name and add new property to dp */
 	ap = findAtt (root, "name", errmsg);
@@ -455,11 +600,16 @@ INDI_P * INDI_D::addProperty (XMLEle *root, char errmsg[])
 
 	if (findProp (valuXMLAtt(ap)))
 	{
-	    sprintf (errmsg, "INDI: <%s %s %s> already exists", tagXMLEle(root),
+	    snprintf (errmsg, ERRMSG_SIZE, "INDI: <%.64s %.64s %.64s> already exists.\n", tagXMLEle(root),
 							name.ascii(), valuXMLAtt(ap));
 	    return NULL;
 	}
 
+	/* Remove Vertical spacer from group layout, this is done everytime
+	  * a new property arrives. The spacer is then appended to the end of the 
+	  * properties */
+	pg->propertyLayout->removeItem(pg->VerticalSpacer);
+	
 	pp = new INDI_P(pg, QString(valuXMLAtt(ap)));
 
 	/* init state */
@@ -472,7 +622,7 @@ INDI_P * INDI_D::addProperty (XMLEle *root, char errmsg[])
 
 	if (crackLightState (valuXMLAtt(ap), &pp->state) < 0)
 	{
-	    sprintf (errmsg, "INDI: <%s> bogus state %s for %s %s",
+	    snprintf (errmsg, ERRMSG_SIZE, "INDI: <%.64s> bogus state %.64s for %.64s %.64s",
 				tagXMLEle(root), valuXMLAtt(ap), pp->pg->dp->name.ascii(), pp->name.ascii());
 	    delete(pp);
 	    return (NULL);
@@ -524,7 +674,7 @@ INDI_G *  INDI_D::findGroup (QString grouptag, int create, char errmsg[])
         return curGroup;
   }
 
-  sprintf (errmsg, "INDI: group %s not found in %s", grouptag.ascii(), name.ascii());
+  snprintf (errmsg, ERRMSG_SIZE, "INDI: group %.64s not found in %.64s", grouptag.ascii(), name.ascii());
   return NULL;
 }
 
@@ -539,7 +689,7 @@ INDI_G *  INDI_D::findGroup (QString grouptag, int create, char errmsg[])
 
 	ap = findXMLAtt(root, "perm");
 	if (!ap) {
-	    sprintf (errmsg, "INDI: <%s %s %s> missing attribute 'perm'",
+	    snprintf (errmsg, ERRMSG_SIZE,"INDI: <%.64s %.64s %.64s> missing attribute 'perm'",
 					tagXMLEle(root), pp->pg->dp->name.ascii(), pp->name.ascii());
 	    return (-1);
 	}
@@ -550,7 +700,7 @@ INDI_G *  INDI_D::findGroup (QString grouptag, int create, char errmsg[])
 	else if (!strcmp(valuXMLAtt(ap), "rw") || !strcmp(valuXMLAtt(ap), "w"))
 	    *permp = PP_RW;
 	else {
-	    sprintf (errmsg, "INDI: <%s> unknown perm %s for %s %s",
+	    snprintf (errmsg, ERRMSG_SIZE, "INDI: <%.64s> unknown perm %.64s for %.64s %.64s",
 				tagXMLEle(root), valuXMLAtt(ap), pp->pg->dp->name.ascii(), pp->name.ascii());
 	    return (-1);
 	}
@@ -707,7 +857,7 @@ int INDI_D::buildSwitchesGUI (XMLEle *root, char errmsg[])
 	}
 
 	/* decide GUI. might use MENU if OneOf but too many for button array */
-	if (!strcmp (valuXMLAtt(ap), "OneOfMany"))
+	if (!strcmp (valuXMLAtt(ap), "OneOfMany") || !strcmp (valuXMLAtt(ap), "AtMostOne"))
 	{
 	    /* count number of switches -- make menu if too many */
 	    for ( ep = nextXMLEle(root, 1) , n = 0 ; ep != NULL; ep = nextXMLEle(root, 0))
@@ -719,7 +869,11 @@ int INDI_D::buildSwitchesGUI (XMLEle *root, char errmsg[])
 	        pp->guitype = PG_MENU;
 		err = pp->buildMenuGUI (root, errmsg);
 		if (err < 0)
+		{
 		    delete(pp);
+		    pp=0;
+		    return err;
+		}
 		    
 		pp->pg->addProperty(pp);
 		return (err);
@@ -730,7 +884,11 @@ int INDI_D::buildSwitchesGUI (XMLEle *root, char errmsg[])
 	    
 	    err = pp->buildSwitchesGUI(root, errmsg);
 	    if (err < 0)
+	    {
 	      delete (pp);
+	      pp=0;
+	      return err;
+	    }
 	      
 	    pp->pg->addProperty(pp);
 	    return (err);
@@ -743,13 +901,17 @@ int INDI_D::buildSwitchesGUI (XMLEle *root, char errmsg[])
 	    
 	    err = pp->buildSwitchesGUI(root, errmsg);
 	    if (err < 0)
+	    {
 	      delete (pp);
+	      pp=0;
+	      return err;
+	    }
 	      
 	    pp->pg->addProperty(pp);
 	    return (err);
 	}
 	
-	sprintf (errmsg, "INDI: <%s> unknown rule %s for %s %s",
+	snprintf (errmsg, ERRMSG_SIZE, "INDI: <%.64s> unknown rule %.64s for %.64s %.64s",
 				tagXMLEle(root), valuXMLAtt(ap), name.ascii(), pp->name.ascii());
 	    
 	delete(pp);
@@ -781,4 +943,57 @@ int INDI_D::buildLightsGUI (XMLEle *root, char errmsg[])
 	return (0);
 }
 
+/* build GUI for a BLOB GUI.
+ * return 0 if ok, else -1 with reason in errmsg[] */
+int INDI_D::buildBLOBGUI  (XMLEle *root, char errmsg[])
+{
+  INDI_P *pp;
+  PPerm p;
+
+  // build a new property
+  pp = addProperty (root, errmsg);
+  if (!pp)
+    return (-1);
+
+  /* get the permission, it will determine layout issues */
+  if (findPerm (pp, root, &p, errmsg))
+  {
+    delete(pp);
+    return (-1);
+  }
+	 
+  /* we know it will be a number GUI */
+  pp->perm = p;
+  pp->guitype = PG_BLOB;
+	
+  if (pp->buildBLOBGUI(root, errmsg) < 0)
+  {
+    delete (pp);
+    return (-1);
+  }
+	
+  pp->pg->addProperty(pp);
+  return (0);
+}
+
+INDI_E * INDI_D::findElem(QString name)
+{
+  INDI_G *grp;
+  INDI_P *prop;
+  INDI_E *el;
+  
+  for (grp = gl.first(); grp != NULL; grp = gl.next())
+	{
+	   for (prop = grp->pl.first(); prop != NULL; prop = grp->pl.next())
+	   {
+	     el = prop->findElement(name);
+	     if (el != NULL) return el;
+	   }
+	 }
+	 
+  return NULL;
+
+}
+
+
 #include "indidevice.moc"
