@@ -22,9 +22,11 @@
  class KStars;
  class SkyObject;
  class StreamWG;
+ class CCDPreviewWG;
  class QSocketNotifier;
  class KProgressDialog;
  class KDirLister;
+ class SkyObject;
  
  
  /* This class implmements standard properties on the device level*/
@@ -39,26 +41,23 @@
    INDI_D      		*dp;			/* associated device */
 
    StreamWG             *streamWindow;
+   CCDPreviewWG    *CCDPreviewWindow;
    SkyObject   		*currentObject;
    QTimer      		*devTimer;	
    KProgressDialog      *downloadDialog;
    
     
-   enum DTypes { DATA_FITS, DATA_STREAM, DATA_OTHER };
+   enum DTypes { DATA_FITS, DATA_STREAM, DATA_OTHER, DATA_CCDPREVIEW };
    
    void setTextValue(INDI_P *pp);
    void setLabelState(INDI_P *pp);
    void registerProperty(INDI_P *pp);
+   void handleBLOB(unsigned char *buffer, int bufferSize, QString dataFormat);
     
-   /* Data channel */
-   void establishDataChannel(QString host, int port);
-   void allocateCompressedBuffer();
-   void allocateStreamBuffer();
-   
    /* Device options */
    void initDeviceOptions();
    void handleDevCounter();
-   bool handleNonSidereal(SkyObject *o);
+   bool handleNonSidereal();
    void streamDisabled();
    
    
@@ -71,11 +70,6 @@
    
    int                  dataType;
    int 			initDevCounter;
-   int 			streamFD;
-   unsigned int 	totalCompressedBytes, totalBytes;
-   unsigned char        *streamBuffer;
-   unsigned char	*compressedBuffer;
-   QSocketNotifier 	*sNotifier;
    QString		dataExt;
    LilXML		*parser;
    
@@ -84,9 +78,9 @@
    bool			batchMode;
    bool			ISOMode;
    KDirLister           *seqLister;
+   SkyObject		*telescopeSkyObject;
    
    public slots:
-   void streamReceived();
    void timerDone();
    
    protected slots:
