@@ -1,7 +1,7 @@
 /***************************************************************************
  *   ktouchtrainer.cpp                                                     *
  *   -----------------                                                     *
- *   Copyright (C) 2000 by Håvard Frøiland, 2004 by Andreas Nicolai        *
+ *   Copyright (C) 2000 by Håvard Frøiland, 2006 by Andreas Nicolai        *
  *   ghorwin@users.sourceforge.net                                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -73,6 +73,9 @@ void KTouchTrainer::gotoFirstLine() {
 // ----------------------------------------------------------------------------
 
 void KTouchTrainer::keyPressed(QChar key) {
+	// NOTE : In this function we need to distinguish between left and right
+	//        typing. Use the config setting Prefs::right2LeftTyping() for that.
+
 	if (m_trainingPaused)  continueTraining();
     if (m_teacherText==m_studentText) {
         // if already at end of line, don't add more chars
@@ -81,13 +84,21 @@ void KTouchTrainer::keyPressed(QChar key) {
         return;
     }
 	// remember length of student text without added character
-    unsigned int len = m_studentText.length();
-	// don´t allow excessive amounts of characters per line
-	if (len > 150) {
+    unsigned int old_student_text_len = m_studentText.length();
+    // compose new student text depending in typing direction
+    QString new_student_text = m_studentText;
+    if (Prefs::right2LeftTyping())
+        new_student_text = key + new_student_text;
+    else
+        new_student_text += key;
+
+    // don´t allow excessive amounts of characters per line
+	if (!m_slideLineWidget->canAddCharacter(new_student_text)) {
         if (Prefs::beepOnError())   QApplication::beep();
         return;
 	}
-    m_studentText += key;
+    // store the new student text
+    m_studentText = new_student_text;
     // we need to find out, if the key was correct or not
     if (studentLineCorrect())
 		statsAddCorrectChar(key);	// ok, all student text is correct
@@ -96,8 +107,24 @@ void KTouchTrainer::keyPressed(QChar key) {
         if (Prefs::beepOnError())  QApplication::beep();
         // check if the key is the first wrong key that was mistyped. Only then add it
 		// to the wrong char statistics.
-		if (m_teacherText.left(len)==m_studentText.left(len) && m_teacherText.length()>len) {
-			statsAddWrongChar(m_teacherText[len]);  // add the key the student ought to press
+        if (Prefs::right2LeftTyping()) {
+			if (m_teacherText.right(old_student_text_len)==m_studentText.right(old_student_text_len) &&
+			    m_teacherText.length() > old_student_text_len)
+			{
+				// add the key the student ought to press to the wrong character stats
+				int next_key_index = m_teacherText.length() - old_student_text_len;
+//				kdDebug() << "Wrong key = " << m_teacherText[next_key_index] << endl;
+				statsAddWrongChar( m_teacherText[next_key_index] );
+			}
+        }
+        else {
+			if (m_teacherText.left(old_student_text_len)==m_studentText.left(old_student_text_len) &&
+				m_teacherText.length() > old_student_text_len)
+			{
+				// add the key the student ought to press to the wrong character stats
+				int next_key_index = old_student_text_len;
+				statsAddWrongChar( m_teacherText[next_key_index] );
+			}
 		}
 		/// \todo 	Implement option whether subsequent mistyped keys should be remembered as missed
 		///			keys as well.
@@ -300,8 +327,14 @@ void KTouchTrainer::storeTrainingStatistics() {
 // ----------------------------------------------------------------------------
 
 bool KTouchTrainer::studentLineCorrect() const { 
+    if (m_teacherText.isEmpty())
+        return m_studentText.isEmpty();
 	unsigned int len = m_studentText.length();
-	return (m_teacherText.left(len)==m_studentText && m_teacherText.length()>=len); 
+    // different check for left and right writing
+    if (Prefs::right2LeftTyping())
+        return m_teacherText.right(len)==m_studentText && m_teacherText.length()>=len;
+    else
+	   return (m_teacherText.left(len)==m_studentText && m_teacherText.length()>=len); 
 }
 // ----------------------------------------------------------------------------
 
