@@ -274,7 +274,7 @@ void KErfassen::slotDlgEnde()
 	{
 		int willErfassen  = KMessageBox::warningYesNoCancel(this,
 					i18n("The current verb is not in the list yet.\nDo you want to add it?"),
-					i18n("Warning"));
+					i18n("Warning"),i18n("Add"),i18n("Do Not Add"));
 		switch (willErfassen)
 		{
 		case KMessageBox::Yes : // YES
@@ -421,7 +421,7 @@ spanishVerbList::Iterator KErfassen::verbErfassen()
 			// Das Verb wurde in der Liste gefunden.
 			switch (KMessageBox::warningYesNoCancel(this,
 					i18n("The current verb is already in the list.\nDo you want to replace it?\nIf you do not want to change the list press 'Cancel'."),
-					i18n("Warning")))
+					i18n("Warning"), i18n("Replace"), i18n("Do Not Replace")))
 			{
 			case KMessageBox::Yes:	// the verb at the position is replaced by v.
 				*it = v;
